@@ -136,9 +136,9 @@ class M3U8(object):
         ('discontinuity_sequence', 'discontinuity_sequence')
     )
 
-    def __init__(self, content=None, base_path=None, base_uri=None, strict=False):
+    def __init__(self, content=None, base_path=None, base_uri=None, strict=False, custom_tags_parser=None):
         if content is not None:
-            self.data = parse(content, strict)
+            self.data = parse(content, strict, custom_tags_parser)
         else:
             self.data = {}
         self._base_uri = base_uri
@@ -255,14 +255,19 @@ class M3U8(object):
         if self.target_duration:
             output.append('#EXT-X-TARGETDURATION:' +
                           int_or_float_to_string(self.target_duration))
-        if self.program_date_time is not None:
-            output.append('#EXT-X-PROGRAM-DATE-TIME:' + format_date_time(self.program_date_time))
         if not (self.playlist_type is None or self.playlist_type == ''):
             output.append('#EXT-X-PLAYLIST-TYPE:%s' % str(self.playlist_type).upper())
         if self.start:
             output.append(str(self.start))
         if self.is_i_frames_only:
             output.append('#EXT-X-I-FRAMES-ONLY')
+        if self.segment_map:
+            map_output = []
+            if self.segment_map.get('uri'):
+                map_output.append('URI=' + quoted(self.segment_map['uri']))
+            if self.segment_map.get('byterange'):
+                map_output.append('BYTERANGE=' + self.segment_map['byterange'])
+            output.append('#EXT-X-MAP:' + ','.join(map_output))
         if self.is_variant:
             if self.media:
                 output.append(str(self.media))
@@ -306,9 +311,15 @@ class Segment(BasePathMixin):
       title attribute from EXTINF parameter
 
     `program_date_time`
-      Returns the EXT-X-PROGRAM-DATE-TIME as a datetime
+      Returns the EXT-X-PROGRAM-DATE-TIME as a datetime. This field is only set
+      if EXT-X-PROGRAM-DATE-TIME exists for this segment
       http://tools.ietf.org/html/draft-pantos-http-live-streaming-07#section-3.3.5
 
+    `current_program_date_time`
+      Returns a datetime of this segment, either the value of `program_date_time`
+      when EXT-X-PROGRAM-DATE-TIME is set or a calculated value based on previous
+      segments' EXT-X-PROGRAM-DATE-TIME and EXTINF values
+
     `discontinuity`
       Returns a boolean indicating if a EXT-X-DISCONTINUITY tag exists
       http://tools.ietf.org/html/draft-pantos-http-live-streaming-13#section-3.4.11
@@ -335,15 +346,17 @@ class Segment(BasePathMixin):
       Key used to encrypt the segment (EXT-X-KEY)
     '''
 
-    def __init__(self, uri, base_uri, program_date_time=None, duration=None,
-                 title=None, byterange=None, cue_out=False, discontinuity=False, key=None,
-                 scte35=None, scte35_duration=None, keyobject=None):
+    def __init__(self, uri, base_uri, program_date_time=None, current_program_date_time=None,
+                 duration=None, title=None, byterange=None, cue_out=False,
+                 discontinuity=False, key=None, scte35=None, scte35_duration=None,
+                 keyobject=None):
         self.uri = uri
         self.duration = duration
         self.title = title
         self.base_uri = base_uri
         self.byterange = byterange
         self.program_date_time = program_date_time
+        self.current_program_date_time = current_program_date_time
         self.discontinuity = discontinuity
         self.cue_out = cue_out
         self.scte35 = scte35
@@ -364,9 +377,9 @@ class Segment(BasePathMixin):
 
         if self.discontinuity:
             output.append('#EXT-X-DISCONTINUITY\n')
-            if self.program_date_time:
-                output.append('#EXT-X-PROGRAM-DATE-TIME:%s\n' %
-                              format_date_time(self.program_date_time))
+        if self.program_date_time:
+            output.append('#EXT-X-PROGRAM-DATE-TIME:%s\n' %
+                          format_date_time(self.program_date_time))
         if self.cue_out:
             output.append('#EXT-X-CUE-OUT-CONT\n')
         output.append('#EXTINF:%s,' % int_or_float_to_string(self.duration))
@@ -497,7 +510,8 @@ class Playlist(BasePathMixin):
             average_bandwidth=stream_info.get('average_bandwidth'),
             program_id=stream_info.get('program_id'),
             resolution=resolution_pair,
-            codecs=stream_info.get('codecs')
+            codecs=stream_info.get('codecs'),
+            frame_rate=stream_info.get('frame_rate')
         )
         self.media = []
         for media_type in ('audio', 'video', 'subtitles'):
@@ -522,6 +536,8 @@ class Playlist(BasePathMixin):
             res = str(self.stream_info.resolution[
                       0]) + 'x' + str(self.stream_info.resolution[1])
             stream_inf.append('RESOLUTION=' + res)
+        if self.stream_info.frame_rate:
+            stream_inf.append('FRAME-RATE=%.5g' % self.stream_info.frame_rate)
         if self.stream_info.codecs:
             stream_inf.append('CODECS=' + quoted(self.stream_info.codecs))
 
@@ -573,7 +589,8 @@ class IFramePlaylist(BasePathMixin):
             average_bandwidth=None,
             program_id=iframe_stream_info.get('program_id'),
             resolution=resolution_pair,
-            codecs=iframe_stream_info.get('codecs')
+            codecs=iframe_stream_info.get('codecs'),
+            frame_rate=None
         )
 
     def __str__(self):
@@ -598,7 +615,7 @@ class IFramePlaylist(BasePathMixin):
 
 StreamInfo = namedtuple(
     'StreamInfo',
-    ['bandwidth', 'closed_captions', 'average_bandwidth', 'program_id', 'resolution', 'codecs', 'audio', 'video', 'subtitles']
+    ['bandwidth', 'closed_captions', 'average_bandwidth', 'program_id', 'resolution', 'codecs', 'audio', 'video', 'subtitles', 'frame_rate']
 )
 
 
