@@ -35,8 +35,7 @@ class ParseError(Exception):
         return 'Syntax error in manifest on line %d: %s' % (self.lineno, self.line)
 
 
-
-def parse(content, strict=False):
+def parse(content, strict=False, custom_tags_parser=None):
     '''
     Given a M3U8 playlist content returns a dictionary with all data found
     '''
@@ -75,11 +74,15 @@ def parse(content, strict=False):
         elif line.startswith(protocol.ext_x_media_sequence):
             _parse_simple_parameter(line, data, int)
 
+        elif line.startswith(protocol.ext_x_discontinuity_sequence):
+            _parse_simple_parameter(line, data, int)
+
         elif line.startswith(protocol.ext_x_program_date_time):
             _, program_date_time = _parse_simple_parameter_raw_value(line, cast_date_time)
             if not data.get('program_date_time'):
                 data['program_date_time'] = program_date_time
             state['current_program_date_time'] = program_date_time
+            state['program_date_time'] = program_date_time
 
         elif line.startswith(protocol.ext_x_discontinuity):
             state['discontinuity'] = True
@@ -150,8 +153,8 @@ def parse(content, strict=False):
 
         # Comments and whitespace
         elif line.startswith('#'):
-            # comment
-            pass
+            if callable(custom_tags_parser):
+                custom_tags_parser(line, data, lineno)
 
         elif line.strip() == '':
             # blank lines are legal
@@ -198,8 +201,10 @@ def _parse_extinf(line, data, state, lineno, strict):
 
 def _parse_ts_chunk(line, data, state):
     segment = state.pop('segment')
+    if state.get('program_date_time'):
+        segment['program_date_time'] = state.pop('program_date_time')
     if state.get('current_program_date_time'):
-        segment['program_date_time'] = state['current_program_date_time']
+        segment['current_program_date_time'] = state['current_program_date_time']
         state['current_program_date_time'] += datetime.timedelta(seconds=segment['duration'])
     segment['uri'] = line
     segment['cue_out'] = state.pop('cue_out', False)
@@ -234,10 +239,11 @@ def _parse_attribute_list(prefix, line, atribute_parser):
 def _parse_stream_inf(line, data, state):
     data['is_variant'] = True
     data['media_sequence'] = None
-    atribute_parser = remove_quotes_parser('codecs', 'audio', 'video', 'subtitles')
+    atribute_parser = remove_quotes_parser('codecs', 'audio', 'video', 'subtitles', 'closed_captions')
     atribute_parser["program_id"] = int
     atribute_parser["bandwidth"] = lambda x: int(float(x))
     atribute_parser["average_bandwidth"] = int
+    atribute_parser["frame_rate"] = float
     state['stream_info'] = _parse_attribute_list(protocol.ext_x_stream_inf, line, atribute_parser)
 
 
@@ -320,7 +326,7 @@ def _parse_cueout_start(line, state, prevline):
 
 
 def string_to_lines(string):
-    return string.strip().replace('\r\n', '\n').split('\n')
+    return string.strip().splitlines()
 
 
 def remove_quotes_parser(*attrs):
@@ -338,7 +344,7 @@ def remove_quotes(string):
 
     '''
     quotes = ('"', "'")
-    if string and string[0] in quotes and string[-1] in quotes:
+    if string.startswith(quotes) and string.endswith(quotes):
         return string[1:-1]
     return string
 
@@ -348,4 +354,5 @@ def normalize_attribute(attribute):
 
 
 def is_url(uri):
-    return re.match(r'https?://', uri) is not None
+    return uri.startswith(('https://', 'http://'))
+
