@@ -538,6 +538,24 @@ g_50123.ts
 
 '''
 
+PLAYLIST_WITH_PROGRAM_DATE_TIME_WITHOUT_DISCONTINUITY = '''
+#EXTM3U
+#EXT-X-VERSION:3
+#EXT-X-TARGETDURATION:6
+#EXT-X-PLAYLIST-TYPE:EVENT
+#EXT-X-MEDIA-SEQUENCE:50
+#EXT-X-PROGRAM-DATE-TIME:2019-06-10T00:05:00.000Z
+#EXTINF:6.000,
+manifest_1_50.ts?m=1559946393
+#EXT-X-PROGRAM-DATE-TIME:2019-06-10T00:05:06.000Z
+#EXTINF:6.000,
+manifest_1_51.ts?m=1559946393
+#EXT-X-PROGRAM-DATE-TIME:2019-06-10T00:05:12.000Z
+#EXTINF:6.000,
+manifest_1_52.ts?m=1559946393
+#EXT-X-ENDLIST
+'''
+
 CUE_OUT_PLAYLIST = '''
 #EXTM3U
 #EXT-X-TARGETDURATION:10
@@ -673,6 +691,14 @@ http://media.example.com/entire.ts
 #EXT-X-ENDLIST
 '''
 
+SIMPLE_PLAYLIST_WITH_CUSTOM_TAGS = '''#EXTM3U
+#EXT-X-MOVIE: million dollar baby
+#EXT-X-TARGETDURATION:5220
+#EXTINF:5220,
+http://media.example.com/entire.ts
+#EXT-X-ENDLIST
+'''
+
 RELATIVE_PLAYLIST_FILENAME = abspath(join(dirname(__file__), 'playlists/relative-playlist.m3u8'))
 
 RELATIVE_PLAYLIST_URI = TEST_HOST + '/path/to/relative-playlist.m3u8'
@@ -681,4 +707,16 @@ CUE_OUT_PLAYLIST_FILENAME = abspath(join(dirname(__file__), 'playlists/cue_out.m
 
 CUE_OUT_PLAYLIST_URI = TEST_HOST + '/path/to/cue_out.m3u8'
 
+VARIANT_PLAYLIST_WITH_FRAME_RATE = '''
+#EXTM3U
+#EXT-X-STREAM-INF:PROGRAM-ID=1,BANDWIDTH=1280000,FRAME-RATE=25
+http://example.com/low.m3u8
+#EXT-X-STREAM-INF:PROGRAM-ID=1,BANDWIDTH=2560000,FRAME-RATE=50
+http://example.com/mid.m3u8
+#EXT-X-STREAM-INF:PROGRAM-ID=1,BANDWIDTH=7680000,FRAME-RATE=60
+http://example.com/hi.m3u8
+#EXT-X-STREAM-INF:PROGRAM-ID=1,BANDWIDTH=65000,FRAME-RATE=12.5,CODECS="mp4a.40.5,avc1.42801e"
+http://example.com/audio-only.m3u8
+'''
+
 del abspath, dirname, join
